<?php
namespace bhl_bind_ecslite;
use Exception;

function supported_tokens()
{
  return [
    'bhl_ecslite_component',
    'bhl_ecslite_component_entity',
  ];
}

class BindEclLitePlugin implements \bhl_bind\BindPlugin
{
  private $units = array();

  function init(\mtgMetaInfo $meta, \Twig\Environment $twig)
  {
    $loader = $twig->getLoader();
    if($loader instanceof \Twig\Loader\FilesystemLoader)
    {
      $loader->setPaths(
        array_merge(
          $loader->getPaths(), 
          [__DIR__ . "/../tpl"]
        )
      );
    }

    //let's collect units we're responsible for
    foreach($meta->getUnits() as $u)
    {
      if($u->object->hasToken('bhl_ecslite_component'))
      {
        $this->units[] = $u;
      }
    }

    //let's remove our units from the original collection
    foreach($this->units as $u)
    {
      $meta->delUnit($u);
    }
  }

  function codegenDecls(\Twig\Environment $twig)
  {
    return '';
  }

  function codegenRegisterDecls(\Twig\Environment $twig)
  {
    $sliced = \bhl_bind\slice_units($this->units, 20);

    $code = '';

    $code .= "#if !BHL_FRONT\n";
    $code .= "public static bool TryUnpackEcsEntity(ValStack stack, out EcsWorld world, out int id)\n";
    $code .= "{\n";
    $code .= "  EcsPackedEntityWithWorld e = default;\n";
    $code .= "  stack.PopRelease().Decode(ref e);\n\n";
    $code .= "  return e.Unpack(out world, out id);\n";
    $code .= "}\n\n";

    $code .= "public static void UnpackEcsEntity(ValStack stack, out EcsWorld world, out int id)\n";
    $code .= "{\n";
    $code .= "  EcsPackedEntityWithWorld e = default;\n";
    $code .= "  stack.PopRelease().Decode(ref e);\n\n";
    $code .= "  if(e.Unpack(out world, out id) == false)\n";
    $code .= "    throw new Exception(\"No entity found\");\n";
    $code .= "}\n\n";

    $code .= "#endif\n";

    foreach($sliced as $idx => $units)
    {
      $code .= "public static void Register_Ecslite_$idx(Types types)\n";
      $code .= "{\n";

      foreach($units as $u)
      {
        if($u->object->hasToken('bhl_ecslite_component'))
          $code .= $twig->render('bind_ecslite.twig', ['unit' => $u->object]);
      }
      $code .= "\n}\n";
    }

    return $code;
  }

  function codegenRegister(\Twig\Environment $twig)
  {
    $sliced = \bhl_bind\slice_units($this->units, 20);

    $code = '';
    foreach($sliced as $idx => $units)
      $code .= "Register_Ecslite_$idx(types);\n";

    return $code;
  }
}
